#!/usr/bin/env node
'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _utils = require('./utils');

var _compiler = require('../compiler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright 2016-present, Sitegen team
 */

var cwd = process.cwd();

var args = (0, _utils.parse)(function (p) {
  return p.option('--inline-css', 'Inline CSS bundle into HTML page').option('-o, --output [output]', 'Output directory relative to entry [default: output]');
});

var _args$args = (0, _slicedToArray3.default)(args.args, 1);

var entry = _args$args[0];


if (args.output) {
  args.output = _path2.default.join(cwd, args.output);
} else {
  args.output = _path2.default.join(_path2.default.dirname(entry), 'output');
}

var debug = (0, _debug2.default)('sitegen:cmd:build');

var compileContent = (0, _compiler.createCompiler)({
  entry: entry,
  output: args.output,
  env: 'content',
  inlineCSS: args.inlineCss,
  publicPath: args.publicPath
});

var compileAssets = (0, _compiler.createCompiler)({
  entry: entry,
  output: args.output,
  env: 'production',
  inlineCSS: args.inlineCss,
  publicPath: args.publicPath
});

function onFinish(message, err, stats) {
  if (err) {
    (0, _utils.error)(err);
  } else if (stats.compilation.errors.length > 0) {
    stats.compilation.errors.map(function (err) {
      return (0, _utils.error)(err, null);
    });
  } else {
    debug(message);
  }
}

compileContent.run(onFinish.bind(null, 'content build complete'));
compileAssets.run(onFinish.bind(null, 'assets build complete'));
