'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.createCompiler = createCompiler;
exports.configureCompiler = configureCompiler;

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _reactRouter = require('react-router');

var _eval = require('eval');

var _eval2 = _interopRequireDefault(_eval);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _route = require('./route');

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright 2016-present, Sitegen team
 */

var BOOT_LOADER = require.resolve('./loader/boot');
var BABEL_LOADER = require.resolve('babel-loader');
var CSS_LOADER = require.resolve('css-loader');
var URL_LOADER = require.resolve('url-loader');
var STYLE_LOADER = require.resolve('style-loader');
var REACTDOWN_LOADER = require.resolve('reactdown/webpack');
var CSS_COMPONENT_LOADER = require.resolve('react-css-components/webpack');

var ES2015_BABEL_PRESET = require.resolve('babel-preset-es2015');
var STAGE_1_BABEL_PRESET = require.resolve('babel-preset-stage-1');
var REACT_BABEL_PRESET = require.resolve('babel-preset-react');
var REACT_HOT_BABEL_PLUGIN = require.resolve('react-hot-loader/babel');

function createCompiler(config) {
  config = configureCompiler(config);
  return (0, _webpack2.default)(config);
}

function configureCompiler(_ref) {
  var entry = _ref.entry;
  var output = _ref.output;
  var env = _ref.env;
  var inlineCSS = _ref.inlineCSS;

  var __DEBUG__ = env === 'production' ? undefined : (0, _stringify2.default)(process.env.DEBUG);

  var BABEL_LOADER_CONFIG = {
    test: /\.js$/,
    exclude: /node_modules/,
    loader: BABEL_LOADER
  };

  var IMG_LOADER_CONFIG = {
    test: /\.(png|jpg|jpeg|gif|ico)$/,
    exclude: /node_modules/,
    loader: (0, _config.configureLoader)({ loader: URL_LOADER, query: { limit: 10000 } })
  };

  var CSS_LOADER_CONFIG = env === 'development' ? {
    test: /\.css$/,
    exclude: /\.(module|component)\.css$/,
    loaders: [STYLE_LOADER, CSS_LOADER]
  } : {
    test: /\.css$/,
    exclude: /\.(module|component)\.css$/,
    loader: _extractTextWebpackPlugin2.default.extract(STYLE_LOADER, CSS_LOADER)
  };

  var CSS_MODULE_LOADER_CONFIG = env === 'development' ? {
    test: /\.module\.css$/,
    loaders: [STYLE_LOADER, CSS_LOADER + '?modules']
  } : {
    test: /\.module\.css$/,
    loader: _extractTextWebpackPlugin2.default.extract(STYLE_LOADER, CSS_LOADER + '?modules')
  };

  var CSS_COMPONENT_LOADER_CONFIG = env === 'development' ? {
    test: /\.component\.css$/,
    loader: (0, _config.configureLoader)([BABEL_LOADER, {
      loader: CSS_COMPONENT_LOADER,
      query: { loadCSS: (0, _config.configureLoader)([STYLE_LOADER, CSS_LOADER + '?modules']).split('!') }
    }])
  } : {
    test: /\.component\.css$/,
    loader: (0, _config.configureLoader)([BABEL_LOADER, {
      loader: CSS_COMPONENT_LOADER,
      query: { loadCSS: _extractTextWebpackPlugin2.default.extract(STYLE_LOADER, CSS_LOADER + '?modules').split('!') }
    }])
  };

  var REACTDOWN_LOADER_CONFIG = {
    test: /\.md$/,
    loaders: [BABEL_LOADER, REACTDOWN_LOADER]
  };

  return {
    entry: [env === 'development' && 'react-hot-loader/patch', env === 'development' && 'webpack-hot-middleware/client', [BOOT_LOADER, entry].join('!')].filter(Boolean),
    devtool: env === 'development' ? 'cheap-module-source-map' : undefined,
    env: env,
    target: env === 'content' ? 'node' : 'web',
    bail: env === 'content',
    output: {
      path: output || '/build',
      filename: 'bundle.js',
      chunkFilename: 'bundle[name].js',
      publicPath: '/'
    },
    module: {
      loaders: [BABEL_LOADER_CONFIG, CSS_LOADER_CONFIG, IMG_LOADER_CONFIG, CSS_MODULE_LOADER_CONFIG, CSS_COMPONENT_LOADER_CONFIG, REACTDOWN_LOADER_CONFIG]
    },
    babel: {
      presets: [ES2015_BABEL_PRESET, STAGE_1_BABEL_PRESET, REACT_BABEL_PRESET],
      plugins: [env === 'development' && REACT_HOT_BABEL_PLUGIN].filter(Boolean)
    },
    plugins: [new LogProgressPlugin(env), new _webpack2.default.DefinePlugin({
      '__DEBUG__': __DEBUG__,
      'process.env.NODE_ENV': (0, _stringify2.default)(env)
    }), env === 'development' && new _webpack2.default.optimize.OccurenceOrderPlugin(), env === 'development' && new _webpack2.default.HotModuleReplacementPlugin(), env === 'development' && new _webpack2.default.NoErrorsPlugin(), env === 'development' && new PromiseAssetsPlugin({
      name: 'promiseBundle',
      then: evalBundle
    }), (env === 'content' || env === 'production') && new _extractTextWebpackPlugin2.default('bundle.css'), env === 'content' && new RenderStaticPlugin({
      inlineCSS: inlineCSS
    }), env === 'production' && new _webpack2.default.optimize.UglifyJsPlugin({ compress: { warnings: false } })].filter(Boolean)
  };
}

function evalBundle(assets) {
  var source = assetSource(assets['bundle.js']);
  var scope = {
    console: console,
    process: process,
    require: require,
    setTimeout: setTimeout
  };
  return (0, _eval2.default)('module.exports = ' + source, '<boot>', scope);
}

var PromiseAssetsPlugin = function () {
  function PromiseAssetsPlugin(_ref2) {
    var name = _ref2.name;
    var then = _ref2.then;
    (0, _classCallCheck3.default)(this, PromiseAssetsPlugin);

    this.name = name || 'promiseAssets';
    this.then = then || function (assets) {
      return assets;
    };
  }

  (0, _createClass3.default)(PromiseAssetsPlugin, [{
    key: 'apply',
    value: function apply(compiler) {
      var _this = this;

      var pending = void 0;

      var init = function init() {
        pending = _bluebird2.default.pending();
        compiler[_this.name] = pending.promise.then(_this.then);
      };

      init();

      compiler.plugin('done', function (_ref3) {
        var compilation = _ref3.compilation;

        pending.resolve(compilation.assets);
      });
      compiler.plugin('invalid', init);
    }
  }]);
  return PromiseAssetsPlugin;
}();

var RenderStaticPlugin = function () {
  function RenderStaticPlugin(_ref4) {
    var inlineCSS = _ref4.inlineCSS;
    (0, _classCallCheck3.default)(this, RenderStaticPlugin);

    this.inlineCSS = inlineCSS;
  }

  (0, _createClass3.default)(RenderStaticPlugin, [{
    key: 'apply',
    value: function apply(compiler) {
      var _this2 = this;

      compiler.plugin('emit', function (compilation, done) {
        var _evalBundle = evalBundle(compilation.assets);

        var route = _evalBundle.route;
        var Meta = _evalBundle.Meta;
        var Site = _evalBundle.Site;

        var css = compilation.assets['bundle.css'];

        cleanAssets(compilation.assets);

        function addToAssets(path, markup) {
          compilation.assets[routePathToAssetPath(path)] = createAssetFromContents(markup);
        }

        var tasks = [];
        (0, _route.forEachPath)(route, function (path) {
          compiler.debug('rendering path', path);
          tasks.push(_this2.renderPath(route, path, css, Site, Meta).then(function (markup) {
            if (markup) {
              addToAssets(path, markup);
            }
          }, function (error) {
            compiler.debug('error while rendering path', path);
            throw error;
          }));
        });

        _bluebird2.default.all(tasks).then(function () {
          done();
        }, function (err) {
          compilation.errors.push(err);
          done(err);
        });
      });
    }
  }, {
    key: 'renderPath',
    value: function renderPath(route, path, css, Site, Meta) {
      var _this3 = this;

      return new _bluebird2.default(function (resolve, reject) {
        var location = path;
        (0, _reactRouter.match)({ routes: route, location: location }, function (error, redirectLocation, routeProps) {
          if (error) {
            reject(error);
          } else if (!routeProps || redirectLocation) {
            resolve(null);
          } else {
            var innerMarkup = (0, _server.renderToString)(_react2.default.createElement(_reactRouter.RouterContext, routeProps));
            var meta = Meta.rewind();
            var markup = (0, _server.renderToStaticMarkup)(_react2.default.createElement(Site, {
              meta: meta,
              bundle: { js: '/bundle.js', css: !_this3.inlineCSS && '/bundle.css' },
              style: _this3.inlineCSS && assetSource(css),
              content: innerMarkup
            }));
            resolve(markup);
          }
        });
      });
    }
  }]);
  return RenderStaticPlugin;
}();

function assetSource(asset) {
  if (!asset) {
    return null;
  }
  return asset._source ? asset._source.source() : asset.source();
}

function createAssetFromContents(contents) {
  return {
    source: function source() {
      return contents;
    },
    size: function size() {
      return contents.length;
    }
  };
}

function routePathToAssetPath(path) {
  path = path.replace(/^\//, '').replace(/\/$/, '');
  return path + '/index.html';
}

function cleanAssets(assets) {
  (0, _keys2.default)(assets).forEach(function (key) {
    return delete assets[key];
  });
}

var LogProgressPlugin = function () {
  function LogProgressPlugin() {
    var name = arguments.length <= 0 || arguments[0] === undefined ? 'webpack' : arguments[0];
    (0, _classCallCheck3.default)(this, LogProgressPlugin);

    this.name = name;
    this.debug = (0, _debug2.default)('sitegen:compile:' + name);
    this._notifyOnCompile = true;
  }

  (0, _createClass3.default)(LogProgressPlugin, [{
    key: 'apply',
    value: function apply(compiler) {
      compiler.debug = this.debug;
      compiler.plugin('compile', this._onCompile.bind(this));
      compiler.plugin('invalid', this._onInvalid.bind(this));
      compiler.plugin('done', this._onDone.bind(this));
    }
  }, {
    key: '_onDone',
    value: function _onDone(stats) {
      var _this4 = this;

      if (stats.compilation.errors.length > 0) {
        this.debug('compilation failed');
        stats.compilation.errors.forEach(function (error) {
          return _this4.debug(error.message);
        });
      } else {
        this.debug('compilation finished');
      }
    }
  }, {
    key: '_onCompile',
    value: function _onCompile() {
      if (this._notifyOnCompile) {
        this._notifyOnCompile = false;
        this.debug('compilation started');
      }
    }
  }, {
    key: '_onInvalid',
    value: function _onInvalid() {
      this.debug('bundled invalidated, recompiling...');
    }
  }]);
  return LogProgressPlugin;
}();
